@@ -1,11 +1,11 @@
 /* Line completion stuff for GDB, the GNU debugger.
-   Copyright 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,34 +14,35 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "symtab.h"
 #include "gdbtypes.h"
 #include "expression.h"
-#include "filenames.h"		/* for DOSish file names */
+#include "filenames.h"		/* For DOSish file names.  */
+#include "language.h"
+#include "gdb_assert.h"
 
 #include "cli/cli-decode.h"
 
-/* FIXME: This is needed because of lookup_cmd_1().
-   We should be calling a hook instead so we eliminate the CLI dependency. */
+/* FIXME: This is needed because of lookup_cmd_1 ().  We should be
+   calling a hook instead so we eliminate the CLI dependency.  */
 #include "gdbcmd.h"
 
 /* Needed for rl_completer_word_break_characters() and for
    rl_filename_completion_function.  */
-#include <readline/readline.h>
+#include "readline/readline.h"
 
 /* readline defines this.  */
 #undef savestring
 
 #include "completer.h"
 
-/* Prototypes for local functions */
+/* Prototypes for local functions.  */
 static
-char *line_completion_function (const char *text, int matches, char *line_buffer,
+char *line_completion_function (const char *text, int matches, 
+				char *line_buffer,
 				int point);
 
 /* readline uses the word breaks for two things:
@@ -51,19 +52,17 @@ char *line_completion_function (const char *text, int matches, char *line_buffer
    it does affect how much stuff M-? lists.
    (2) If one of the matches contains a word break character, readline
    will quote it.  That's why we switch between
-   gdb_completer_word_break_characters and
+   current_language->la_word_break_characters() and
    gdb_completer_command_word_break_characters.  I'm not sure when
    we need this behavior (perhaps for funky characters in C++ symbols?).  */
 
 /* Variables which are necessary for fancy command line editing.  */
-static char *gdb_completer_word_break_characters =
-" \t\n!@#$%^&*()+=|~`}{[]\"';:?/>.<,-";
 
 /* When completing on command names, we remove '-' from the list of
    word break characters, since we use it in command names.  If the
    readline library sees one in any of the current completion strings,
    it thinks that the string needs to be quoted and automatically supplies
-   a leading quote. */
+   a leading quote.  */
 static char *gdb_completer_command_word_break_characters =
 " \t\n!@#$%^&*()+=|~`}{[]\"';:?/>.<,";
 
@@ -85,16 +84,10 @@ static char *gdb_completer_loc_break_characters = " \t\n*|\"';:?><,";
 
 /* Characters that can be used to quote completion strings.  Note that we
    can't include '"' because the gdb C parser treats such quoted sequences
-   as strings. */
+   as strings.  */
 static char *gdb_completer_quote_characters = "'";
 
-/* Accessor for some completer data that may interest other files. */
-
-char *
-get_gdb_completer_word_break_characters (void)
-{
-  return gdb_completer_word_break_characters;
-}
+/* Accessor for some completer data that may interest other files.  */
 
 char *
 get_gdb_completer_quote_characters (void)
@@ -113,14 +106,14 @@ readline_line_completion_function (const char *text, int matches)
 /* This can be used for functions which don't want to complete on symbols
    but don't want to complete on anything else either.  */
 char **
-noop_completer (char *text, char *prefix)
+noop_completer (struct cmd_list_element *ignore, char *text, char *prefix)
 {
   return NULL;
 }
 
 /* Complete on filenames.  */
 char **
-filename_completer (char *text, char *word)
+filename_completer (struct cmd_list_element *ignore, char *text, char *word)
 {
   int subsequent_name;
   char **return_val;
@@ -135,7 +128,7 @@ filename_completer (char *text, char *word)
   subsequent_name = 0;
   while (1)
     {
-      char *p;
+      char *p, *q;
       p = rl_filename_completion_function (text, subsequent_name);
       if (return_val_used >= return_val_alloced)
 	{
@@ -157,38 +150,38 @@ filename_completer (char *text, char *word)
       /* Like emacs, don't complete on old versions.  Especially useful
          in the "source" command.  */
       if (p[strlen (p) - 1] == '~')
-	continue;
+	{
+	  xfree (p);
+	  continue;
+	}
 
-      {
-	char *q;
-	if (word == text)
-	  /* Return exactly p.  */
-	  return_val[return_val_used++] = p;
-	else if (word > text)
-	  {
-	    /* Return some portion of p.  */
-	    q = xmalloc (strlen (p) + 5);
-	    strcpy (q, p + (word - text));
-	    return_val[return_val_used++] = q;
-	    xfree (p);
-	  }
-	else
-	  {
-	    /* Return some of TEXT plus p.  */
-	    q = xmalloc (strlen (p) + (text - word) + 5);
-	    strncpy (q, word, text - word);
-	    q[text - word] = '\0';
-	    strcat (q, p);
-	    return_val[return_val_used++] = q;
-	    xfree (p);
-	  }
-      }
+      if (word == text)
+	/* Return exactly p.  */
+	return_val[return_val_used++] = p;
+      else if (word > text)
+	{
+	  /* Return some portion of p.  */
+	  q = xmalloc (strlen (p) + 5);
+	  strcpy (q, p + (word - text));
+	  return_val[return_val_used++] = q;
+	  xfree (p);
+	}
+      else
+	{
+	  /* Return some of TEXT plus p.  */
+	  q = xmalloc (strlen (p) + (text - word) + 5);
+	  strncpy (q, word, text - word);
+	  q[text - word] = '\0';
+	  strcat (q, p);
+	  return_val[return_val_used++] = q;
+	  xfree (p);
+	}
     }
 #if 0
   /* There is no way to do this just long enough to affect quote inserting
      without also affecting the next completion.  This should be fixed in
      readline.  FIXME.  */
-  /* Insure that readline does the right thing
+  /* Ensure that readline does the right thing
      with respect to inserting quotes.  */
   rl_completer_word_break_characters = "";
 #endif
@@ -203,7 +196,7 @@ filename_completer (char *text, char *word)
 
    This is intended to be used in commands that set breakpoints etc.  */
 char **
-location_completer (char *text, char *word)
+location_completer (struct cmd_list_element *ignore, char *text, char *word)
 {
   int n_syms = 0, n_files = 0;
   char ** fn_list = NULL;
@@ -237,7 +230,7 @@ location_completer (char *text, char *word)
 	  if (*p == quote_found)
 	    quote_found = 0;
 	  else
-	    break;		/* hit the end of text */
+	    break;		/* Hit the end of text.  */
 	}
 #if HAVE_DOS_BASED_FILE_SYSTEM
       /* If we have a DOS-style absolute file name at the beginning of
@@ -251,7 +244,7 @@ location_completer (char *text, char *word)
 	  colon = p;
 	  symbol_start = p + 1;
 	}
-      else if (strchr (gdb_completer_word_break_characters, *p))
+      else if (strchr (current_language->la_word_break_characters(), *p))
 	symbol_start = p + 1;
     }
 
@@ -287,7 +280,8 @@ location_completer (char *text, char *word)
       list = make_symbol_completion_list (symbol_start, word);
       /* If text includes characters which cannot appear in a file
 	 name, they cannot be asking for completion on files.  */
-      if (strcspn (text, gdb_completer_file_name_break_characters) == text_len)
+      if (strcspn (text, 
+		   gdb_completer_file_name_break_characters) == text_len)
 	fn_list = make_source_files_completion_list (text, text);
     }
 
@@ -337,18 +331,134 @@ location_completer (char *text, char *word)
       /* No completions at all.  As the final resort, try completing
 	 on the entire text as a symbol.  */
       list = make_symbol_completion_list (orig_text, word);
+      xfree (fn_list);
     }
+  else
+    xfree (fn_list);
 
   return list;
 }
 
-/* Complete on command names.  Used by "help".  */
-char **
-command_completer (char *text, char *word)
+/* Helper for expression_completer which recursively counts the number
+   of named fields and methods in a structure or union type.  */
+static int
+count_struct_fields (struct type *type)
 {
-  return complete_on_cmdlist (cmdlist, text, word);
+  int i, result = 0;
+
+  CHECK_TYPEDEF (type);
+  for (i = 0; i < TYPE_NFIELDS (type); ++i)
+    {
+      if (i < TYPE_N_BASECLASSES (type))
+	result += count_struct_fields (TYPE_BASECLASS (type, i));
+      else if (TYPE_FIELD_NAME (type, i))
+	++result;
+    }
+
+  for (i = TYPE_NFN_FIELDS (type) - 1; i >= 0; --i)
+    {
+      if (TYPE_FN_FIELDLIST_NAME (type, i))
+	++result;
+    }
+
+  return result;
 }
 
+/* Helper for expression_completer which recursively adds field and
+   method names from TYPE, a struct or union type, to the array
+   OUTPUT.  This function assumes that OUTPUT is correctly-sized.  */
+static void
+add_struct_fields (struct type *type, int *nextp, char **output,
+		   char *fieldname, int namelen)
+{
+  int i;
+  int computed_type_name = 0;
+  char *type_name = NULL;
+
+  CHECK_TYPEDEF (type);
+  for (i = 0; i < TYPE_NFIELDS (type); ++i)
+    {
+      if (i < TYPE_N_BASECLASSES (type))
+	add_struct_fields (TYPE_BASECLASS (type, i), nextp, output,
+			   fieldname, namelen);
+      else if (TYPE_FIELD_NAME (type, i)
+	       && ! strncmp (TYPE_FIELD_NAME (type, i), fieldname, namelen))
+	{
+	  output[*nextp] = xstrdup (TYPE_FIELD_NAME (type, i));
+	  ++*nextp;
+	}
+    }
+
+  for (i = TYPE_NFN_FIELDS (type) - 1; i >= 0; --i)
+    {
+      char *name = TYPE_FN_FIELDLIST_NAME (type, i);
+      if (name && ! strncmp (name, fieldname, namelen))
+	{
+	  if (!computed_type_name)
+	    {
+	      type_name = type_name_no_tag (type);
+	      computed_type_name = 1;
+	    }
+	  /* Omit constructors from the completion list.  */
+	  if (type_name && strcmp (type_name, name))
+	    {
+	      output[*nextp] = xstrdup (name);
+	      ++*nextp;
+	    }
+	}
+    }
+}
+
+/* Complete on expressions.  Often this means completing on symbol
+   names, but some language parsers also have support for completing
+   field names.  */
+char **
+expression_completer (struct cmd_list_element *ignore, char *text, char *word)
+{
+  struct type *type;
+  char *fieldname, *p;
+
+  /* Perform a tentative parse of the expression, to see whether a
+     field completion is required.  */
+  fieldname = NULL;
+  type = parse_field_expression (text, &fieldname);
+  if (fieldname && type)
+    {
+      for (;;)
+	{
+	  CHECK_TYPEDEF (type);
+	  if (TYPE_CODE (type) != TYPE_CODE_PTR
+	      && TYPE_CODE (type) != TYPE_CODE_REF)
+	    break;
+	  type = TYPE_TARGET_TYPE (type);
+	}
+
+      if (TYPE_CODE (type) == TYPE_CODE_UNION
+	  || TYPE_CODE (type) == TYPE_CODE_STRUCT)
+	{
+	  int alloc = count_struct_fields (type);
+	  int flen = strlen (fieldname);
+	  int out = 0;
+	  char **result = (char **) xmalloc ((alloc + 1) * sizeof (char *));
+
+	  add_struct_fields (type, &out, result, fieldname, flen);
+	  result[out] = NULL;
+	  xfree (fieldname);
+	  return result;
+	}
+    }
+  xfree (fieldname);
+
+  /* Commands which complete on locations want to see the entire
+     argument.  */
+  for (p = word;
+       p > text && p[-1] != ' ' && p[-1] != '\t';
+       p--)
+    ;
+
+  /* Not ideal but it is what we used to do before... */
+  return location_completer (ignore, p, word);
+}
 
 /* Here are some useful test cases for completion.  FIXME: These should
    be put in the test suite.  They should be tested with both M-? and TAB.
@@ -372,18 +482,45 @@ command_completer (char *text, char *word)
    "file ../gdb.stabs/we" "ird" (needs to not break word at slash)
  */
 
-/* Generate completions all at once.  Returns a NULL-terminated array
-   of strings.  Both the array and each element are allocated with
-   xmalloc.  It can also return NULL if there are no completions.
+typedef enum
+{
+  handle_brkchars,
+  handle_completions,
+  handle_help
+}
+complete_line_internal_reason;
+
+
+/* Internal function used to handle completions.
+
 
    TEXT is the caller's idea of the "word" we are looking at.
 
    LINE_BUFFER is available to be looked at; it contains the entire text
    of the line.  POINT is the offset in that line of the cursor.  You
-   should pretend that the line ends at POINT.  */
+   should pretend that the line ends at POINT.
 
-char **
-complete_line (const char *text, char *line_buffer, int point)
+   REASON is of type complete_line_internal_reason.
+
+   If REASON is handle_brkchars:
+   Preliminary phase, called by gdb_completion_word_break_characters function,
+   is used to determine the correct set of chars that are word delimiters
+   depending on the current command in line_buffer.
+   No completion list should be generated; the return value should be NULL.
+   This is checked by an assertion in that function.
+
+   If REASON is handle_completions:
+   Main phase, called by complete_line function, is used to get the list
+   of posible completions.
+
+   If REASON is handle_help:
+   Special case when completing a 'help' command.  In this case,
+   once sub-command completions are exhausted, we simply return NULL.
+ */
+
+static char **
+complete_line_internal (const char *text, char *line_buffer, int point,
+			complete_line_internal_reason reason)
 {
   char **list = NULL;
   char *tmp_command, *p;
@@ -397,11 +534,10 @@ complete_line (const char *text, char *line_buffer, int point)
      functions, which can be any string) then we will switch to the
      special word break set for command strings, which leaves out the
      '-' character used in some commands.  */
-
   rl_completer_word_break_characters =
-    gdb_completer_word_break_characters;
+    current_language->la_word_break_characters();
 
-      /* Decide whether to complete on a list of gdb commands or on symbols. */
+  /* Decide whether to complete on a list of gdb commands or on symbols. */
   tmp_command = (char *) alloca (point + 1);
   p = tmp_command;
 
@@ -460,14 +596,16 @@ complete_line (const char *text, char *line_buffer, int point)
 	     This we can deal with.  */
 	  if (result_list)
 	    {
-	      list = complete_on_cmdlist (*result_list->prefixlist, p,
-					  word);
+	      if (reason != handle_brkchars)
+		list = complete_on_cmdlist (*result_list->prefixlist, p,
+					    word);
 	    }
 	  else
 	    {
-	      list = complete_on_cmdlist (cmdlist, p, word);
+	      if (reason != handle_brkchars)
+		list = complete_on_cmdlist (cmdlist, p, word);
 	    }
-	  /* Insure that readline does the right thing with respect to
+	  /* Ensure that readline does the right thing with respect to
 	     inserting quotes.  */
 	  rl_completer_word_break_characters =
 	    gdb_completer_command_word_break_characters;
@@ -489,16 +627,20 @@ complete_line (const char *text, char *line_buffer, int point)
 		{
 		  /* It is a prefix command; what comes after it is
 		     a subcommand (e.g. "info ").  */
-		  list = complete_on_cmdlist (*c->prefixlist, p, word);
+		  if (reason != handle_brkchars)
+		    list = complete_on_cmdlist (*c->prefixlist, p, word);
 
-		  /* Insure that readline does the right thing
-		         with respect to inserting quotes.  */
+		  /* Ensure that readline does the right thing
+		     with respect to inserting quotes.  */
 		  rl_completer_word_break_characters =
 		    gdb_completer_command_word_break_characters;
 		}
+	      else if (reason == handle_help)
+		list = NULL;
 	      else if (c->enums)
 		{
-		  list = complete_on_enum (c->enums, p, word);
+		  if (reason != handle_brkchars)
+		    list = complete_on_enum (c->enums, p, word);
 		  rl_completer_word_break_characters =
 		    gdb_completer_command_word_break_characters;
 		}
@@ -534,7 +676,8 @@ complete_line (const char *text, char *line_buffer, int point)
 			   p--)
 			;
 		    }
-		  list = (*c->completer) (p, word);
+		  if (reason != handle_brkchars)
+		    list = (*c->completer) (c, p, word);
 		}
 	    }
 	  else
@@ -555,14 +698,17 @@ complete_line (const char *text, char *line_buffer, int point)
 		    break;
 		}
 
-	      list = complete_on_cmdlist (result_list, q, word);
+	      if (reason != handle_brkchars)
+		list = complete_on_cmdlist (result_list, q, word);
 
-		  /* Insure that readline does the right thing
-		     with respect to inserting quotes.  */
+	      /* Ensure that readline does the right thing
+		 with respect to inserting quotes.  */
 	      rl_completer_word_break_characters =
 		gdb_completer_command_word_break_characters;
 	    }
 	}
+      else if (reason == handle_help)
+	list = NULL;
       else
 	{
 	  /* There is non-whitespace beyond the command.  */
@@ -575,7 +721,8 @@ complete_line (const char *text, char *line_buffer, int point)
 	    }
 	  else if (c->enums)
 	    {
-	      list = complete_on_enum (c->enums, p, word);
+	      if (reason != handle_brkchars)
+		list = complete_on_enum (c->enums, p, word);
 	    }
 	  else
 	    {
@@ -600,13 +747,51 @@ complete_line (const char *text, char *line_buffer, int point)
 		       p--)
 		    ;
 		}
-	      list = (*c->completer) (p, word);
+	      if (reason != handle_brkchars)
+		list = (*c->completer) (c, p, word);
 	    }
 	}
     }
 
   return list;
 }
+/* Generate completions all at once.  Returns a NULL-terminated array
+   of strings.  Both the array and each element are allocated with
+   xmalloc.  It can also return NULL if there are no completions.
+
+   TEXT is the caller's idea of the "word" we are looking at.
+
+   LINE_BUFFER is available to be looked at; it contains the entire text
+   of the line.
+
+   POINT is the offset in that line of the cursor.  You
+   should pretend that the line ends at POINT.  */
+
+char **
+complete_line (const char *text, char *line_buffer, int point)
+{
+  return complete_line_internal (text, line_buffer, point, handle_completions);
+}
+
+/* Complete on command names.  Used by "help".  */
+char **
+command_completer (struct cmd_list_element *ignore, char *text, char *word)
+{
+  return complete_line_internal (word, text, strlen (text), handle_help);
+}
+
+/* Get the list of chars that are considered as word breaks
+   for the current command.  */
+
+char *
+gdb_completion_word_break_characters (void)
+{
+  char ** list;
+  list = complete_line_internal (rl_line_buffer, rl_line_buffer, rl_point,
+				 handle_brkchars);
+  gdb_assert (list == NULL);
+  return rl_completer_word_break_characters;
+}
 
 /* Generate completions one by one for the completer.  Each time we are
    called return another potential completion to the caller.
@@ -630,23 +815,26 @@ complete_line (const char *text, char *line_buffer, int point)
    free the string.  */
 
 static char *
-line_completion_function (const char *text, int matches, char *line_buffer, int point)
+line_completion_function (const char *text, int matches, 
+			  char *line_buffer, int point)
 {
-  static char **list = (char **) NULL;	/* Cache of completions */
-  static int index;		/* Next cached completion */
+  static char **list = (char **) NULL;	/* Cache of completions.  */
+  static int index;			/* Next cached completion.  */
   char *output = NULL;
 
   if (matches == 0)
     {
       /* The caller is beginning to accumulate a new set of completions, so
          we need to find all of them now, and cache them for returning one at
-         a time on future calls. */
+         a time on future calls.  */
 
       if (list)
 	{
 	  /* Free the storage used by LIST, but not by the strings inside.
-	     This is because rl_complete_internal () frees the strings. */
+	     This is because rl_complete_internal () frees the strings.
+	     As complete_line may abort by calling `error' clear LIST now.  */
 	  xfree (list);
+	  list = NULL;
 	}
       index = 0;
       list = complete_line (text, line_buffer, point);
@@ -656,7 +844,7 @@ line_completion_function (const char *text, int matches, char *line_buffer, int
      dole them out one at a time.  The vector of completions is NULL
      terminated, so after returning the last one, return NULL (and continue
      to do so) each time we are called after that, until a new list is
-     available. */
+     available.  */
 
   if (list)
     {
@@ -674,7 +862,7 @@ line_completion_function (const char *text, int matches, char *line_buffer, int
     /* Make sure the word break characters are set back to normal for the
        next time that readline tries to complete something.  */
     rl_completer_word_break_characters =
-      gdb_completer_word_break_characters;
+      current_language->la_word_break_characters();
 #endif
 
   return (output);
@@ -696,16 +884,16 @@ skip_quoted_chars (char *str, char *quotechars, char *breakchars)
     quotechars = gdb_completer_quote_characters;
 
   if (breakchars == NULL)
-    breakchars = gdb_completer_word_break_characters;
+    breakchars = current_language->la_word_break_characters();
 
   for (scan = str; *scan != '\0'; scan++)
     {
       if (quote_char != '\0')
 	{
-	  /* Ignore everything until the matching close quote char */
+	  /* Ignore everything until the matching close quote char.  */
 	  if (*scan == quote_char)
 	    {
-	      /* Found matching close quote. */
+	      /* Found matching close quote.  */
 	      scan++;
 	      break;
 	    }
@@ -726,7 +914,7 @@ skip_quoted_chars (char *str, char *quotechars, char *breakchars)
 
 /* Skip over the possibly quoted word STR (as defined by the quote
    characters and word break characters used by the completer).
-   Returns pointer to the location after the "word". */
+   Returns pointer to the location after the "word".  */
 
 char *
 skip_quoted (char *str)
